Imports System.Data.SqlClient
Imports System.Reflection
Imports System.EnterpriseServices

' Supply a strong-named assembly.
<Assembly: AssemblyKeyFileAttribute("..\..\TransDemo.snk")> 

Public Class DataAccess

    Public Sub getCustomers(ByRef conn As SqlConnection, ByRef customerDS As Customers)

        ' get customer data from the table and return it as a typed customer dataset
        Dim customerAdapter As New SqlDataAdapter("GetCustomers_sp", conn)

        Try

            With customerAdapter

                ' map the default table name to the table name defined in the typed dataset
                If Not .TableMappings.Contains("Table") Then
                    .TableMappings.Add("Table", customerDS.Customers.TableName)
                End If

                .Fill(customerDS)
                conn.Close()

                .Dispose()

            End With

        Finally

            ' close the connection if still open
            If conn.State = ConnectionState.Open Then
                conn.Close()
            End If

            If Not customerAdapter Is Nothing Then
                customerAdapter.Dispose()
            End If

        End Try

    End Sub

    Public Sub getProducts(ByRef conn As SqlConnection, ByRef productDS As Products)

        ' get products data from the table and return it as a typed product dataset
        Dim productsAdapter As New SqlDataAdapter("GetProducts_sp", conn)

        Try

            With productsAdapter

                ' map the default table name to the table name defined in the typed dataset
                If Not .TableMappings.Contains("Table") Then
                    .TableMappings.Add("Table", productDS.Products.TableName)
                End If

                .Fill(productDS)
                conn.Close()

                .Dispose()

            End With

        Finally

            ' close the connection if still open
            If conn.State = ConnectionState.Open Then
                conn.Close()
            End If

            If Not productsAdapter Is Nothing Then
                productsAdapter.Dispose()
            End If

        End Try

    End Sub

    Public Function newOrder(ByRef conn As SqlConnection, ByVal customerID As String, _
        ByVal productID As Integer, ByVal requestedQty As Integer, ByVal unitPrice As Double, _
        ByVal discount As Double) As Boolean

        ' this method creates new order and order detail records using an automatic transaction 
        Dim newOrderID As Integer
        Dim orderCommand As SqlCommand
        Dim orderDetailCommand As SqlCommand

        Try

            ' set up the command parameters and commandtext properties for the order command
            orderCommand = setupNewOrderCommand(conn, customerID)

            With orderCommand

                ' insert a new order record
                .ExecuteNonQuery()

                ' return the new order id value
                newOrderID = .Parameters("@iOrderID").Value

            End With

            ' set up the command parameters and commandtext properties for the order detail command
            orderDetailCommand = setupNewOrderDetailCommand(conn, newOrderID, productID, _
                unitPrice, requestedQty, discount)

            ' insert a new order detail record
            orderDetailCommand.ExecuteNonQuery()

            Return True

        Finally

            If Not orderCommand Is Nothing Then
                orderCommand.Dispose()
            End If

            If Not orderDetailCommand Is Nothing Then
                orderDetailCommand.Dispose()
            End If

        End Try

    End Function

    Public Function newOrder(ByRef conn As SqlConnection, ByRef trans As SqlTransaction, ByVal customerID As String, _
        ByVal productID As Integer, ByVal requestedQty As Integer, ByVal unitPrice As Double, _
        ByVal discount As Double) As Boolean

        ' this method creates new order and order detail records using an ADO.Net SqlTransaction object
        Dim newOrderID As Integer
        Dim orderCommand As SqlCommand
        Dim orderDetailCommand As SqlCommand

        Try

            ' set up the command parameters and commandtext properties for the order command
            orderCommand = setupNewOrderCommand(conn, customerID)

            With orderCommand

                ' assign the transaction object to the command's transaction property
                .Transaction = trans

                ' insert a new order record
                .ExecuteNonQuery()

                ' return the new order id value
                newOrderID = .Parameters("@iOrderID").Value

            End With

            ' set up the command parameters and commandtext properties for the order detail command
            orderDetailCommand = setupNewOrderDetailCommand(conn, newOrderID, productID, _
                unitPrice, requestedQty, discount)

            With orderDetailCommand

                ' assign the transaction object (same one from above) to the command's transaction property
                .Transaction = trans

                ' insert a new order detail record
                .ExecuteNonQuery()

            End With

            Return True

        Finally

            If Not orderCommand Is Nothing Then
                orderCommand.Dispose()
            End If

            If Not orderDetailCommand Is Nothing Then
                orderDetailCommand.Dispose()
            End If

        End Try

    End Function

    Public Function adjustInventory(ByRef conn As SqlConnection, ByVal requestedQty As Integer, _
        ByVal productID As Integer) As Boolean

        ' this method adjusts the Products table's UnitsInStock column using an automatic transaction
        Dim inventoryCommand As SqlClient.SqlCommand

        Try

            ' set up the command's parameter and connection properties
            inventoryCommand = setupInventoryCommand(conn, requestedQty, productID)

            ' update the table
            inventoryCommand.ExecuteNonQuery()

            Return True

        Finally

            If Not inventoryCommand Is Nothing Then
                inventoryCommand.Dispose()
            End If

        End Try

    End Function

    Public Function adjustInventory(ByRef conn As SqlConnection, ByRef trans As SqlTransaction, _
        ByVal requestedQty As Integer, ByVal productID As Integer) As Boolean

        ' this method adjusts the Products table's UnitsInStock column using a SqlTransaction object
        Dim inventoryCommand As SqlClient.SqlCommand

        Try

            ' set up the command's parameter and connection properties
            inventoryCommand = setupInventoryCommand(conn, requestedQty, productID)

            ' assign the transaction to the command's transaction property. This transaction object is the same 
            ' one assigned to the order and orderdetails commands (see newOrder method)
            inventoryCommand.Transaction = trans

            ' update the table
            inventoryCommand.ExecuteNonQuery()

            Return True

        Finally

            If Not inventoryCommand Is Nothing Then
                inventoryCommand.Dispose()
            End If

        End Try

    End Function

    Public Function checkCreditCard() As Boolean

        ' Randomly determine validity of credit card 
        Dim randomCheck As New Random(DateTime.Now.Millisecond)

        ' simulate an invalid credit card 40% of the time
        Return (randomCheck.Next(0, 9) < 5)

    End Function

    Private Function setupNewOrderCommand(ByRef conn As SqlConnection, _
        ByVal customerID As String) As SqlCommand

        ' set the commandtext, connection and parameter properties for the
        ' order command
        Dim orderCommand As New SqlCommand("InsertOrder_sp", conn)

        With orderCommand

            .CommandType = CommandType.StoredProcedure

            .Parameters.Add("@iOrderID", 0)
            .Parameters("@iOrderID").Direction = ParameterDirection.Output
            .Parameters.Add("@cCustomerID", customerID)

        End With

        Return orderCommand

    End Function

    Private Function setupNewOrderDetailCommand(ByRef conn As SqlConnection, _
        ByVal newOrderID As Integer, ByVal productID As Integer, ByVal unitPrice As Double, _
        ByVal requestedQty As Integer, ByVal discount As Double) As SqlCommand

        ' set the commandtext, connection and parameter properties for the
        ' order details command
        Dim orderDetailCommand As New SqlCommand("AddNewDetail_sp", conn)

        With orderDetailCommand

            .CommandType = CommandType.StoredProcedure

            .Parameters.Add("@iOrderID", newOrderID)
            .Parameters.Add("@iProductID", productID)
            .Parameters.Add("@mnUnitPrice", unitPrice)
            .Parameters.Add("@iQuantity", requestedQty)
            .Parameters.Add("@fDiscount", discount)

        End With

        Return orderDetailCommand

    End Function

    Private Function setupInventoryCommand(ByRef conn As SqlConnection, ByVal requestedQty As Integer, _
        ByVal productID As Integer) As SqlCommand

        ' set the commandtext, connection and parameter properties for the
        ' products command
        Dim inventoryCommand As SqlClient.SqlCommand
        Dim sqlString As String

        sqlString = "UPDATE Products SET UnitsInStock = UnitsInStock - " & requestedQty & _
            " WHERE ProductID = " & productID

        inventoryCommand = New SqlCommand(sqlString, conn)

        Return inventoryCommand

    End Function

End Class
