using System;
using System.Data;
using System.Reflection;
using System.Data.SqlClient;
using System.EnterpriseServices;

// Supply a strong-named assembly.
//[assembly: AssemblyKeyFileAttribute(@"..\..\TransDemo.snk")] 

namespace DataAccess
{
	/// <summary>
	/// Summary description for DataAccess.
	/// </summary>
	public class DataAccess
	{
		public DataAccess()
		{
			//
			// TODO: Add constructor logic here
			//
		}


		public void getCustomers(SqlConnection conn, Customers customerDS) {

			// get customer data from the table and return it as a typed customer dataset
			SqlDataAdapter customerAdapter = new SqlDataAdapter("GetCustomers_sp", conn);

			try {

				// map the default table name to the table name defined in the typed dataset
				if (! customerAdapter.TableMappings.Contains("Table")) 
					customerAdapter.TableMappings.Add("Table", customerDS.CustomersTable.TableName);
		
				customerAdapter.Fill(customerDS);
				conn.Close();

				customerAdapter.Dispose();
			}

			finally {

				// close the connection if still open
				if (conn.State == ConnectionState.Open)
					conn.Close();
				
				if (customerAdapter != null) 
					customerAdapter.Dispose();
			
			}

		}

		
		public void getProducts(SqlConnection conn, Products productDS) {

			// get products data from the table and return it as a typed product dataset
			SqlDataAdapter productsAdapter = new SqlDataAdapter("GetProducts_sp", conn);

			try 
			{

				// map the default table name to the table name defined in the typed dataset
				if (! productsAdapter.TableMappings.Contains("Table"))
					productsAdapter.TableMappings.Add("Table", productDS.ProductsTable.TableName);

				productsAdapter.Fill(productDS);
				conn.Close();

				productsAdapter.Dispose();

			}

			finally {

					// close the connection if still open
				if (conn.State == ConnectionState.Open)
					conn.Close();

				if (productsAdapter != null)
					productsAdapter.Dispose();

			}

		}

		
		public bool newOrder(SqlConnection conn, string customerID, int productID, 
			int requestedQty, double unitPrice, double discount) 
		{

			// this method creates new order and order detail records using a COM+ transaction 
			Int64 newOrderID;
			SqlCommand orderCommand; 
			SqlCommand orderDetailCommand = new SqlCommand(); 

			// set up the command parameters and commandtext properties for the order command
			orderCommand = setupNewOrderCommand(conn, customerID);

			try 
			{
	
				// insert a new order record
				orderCommand.ExecuteNonQuery();

				// return the new order id value
				newOrderID = (Int64) orderCommand.Parameters["@iOrderID"].Value;

				// set up the command parameters and commandtext properties for the order detail command
				orderDetailCommand = setupNewOrderDetailCommand(conn, newOrderID, productID, unitPrice, requestedQty, discount);

				// insert a new order detail record
				orderDetailCommand.ExecuteNonQuery();

				return true;

			}
			finally 
			{

				if ( orderCommand != null ) 
					orderCommand.Dispose();

				if ( orderDetailCommand != null ) 
					orderDetailCommand.Dispose();
		
			}

		}
										   
		
		public bool newOrder(SqlConnection conn, SqlTransaction trans, string customerID, int productID, 
			int requestedQty, double unitPrice, double discount) 
		{

			// this method creates new order and order detail records using an ADO.Net SqlTransaction object
			Int64 newOrderID;
			SqlCommand orderCommand;
			SqlCommand orderDetailCommand = new SqlCommand();

			// set up the command parameters and commandtext properties for the order command
			orderCommand = setupNewOrderCommand(conn, customerID);

			try 
			{

				// assign the transaction object to the command's transaction property
				orderCommand.Transaction = trans;

				// insert a new order record
				orderCommand.ExecuteNonQuery();

				// return the new order id value
				newOrderID =  (Int64) orderCommand.Parameters["@iOrderID"].Value;

				// set up the command parameters and commandtext properties for the order detail command
				orderDetailCommand = setupNewOrderDetailCommand(conn, newOrderID, productID, unitPrice, requestedQty, discount);

				// assign the transaction object (same one from above) to the command's transaction property
				orderDetailCommand.Transaction = trans;

				// insert a new order detail record
				orderDetailCommand.ExecuteNonQuery();

				return true;

			}
			
			finally 
			{

				if ( orderCommand != null )
					orderCommand.Dispose();
		
				if ( orderDetailCommand != null )
					orderDetailCommand.Dispose();
		
			}

		}


		public bool adjustInventory(SqlConnection conn, int requestedQty, int productID) 
		{

			// this method adjusts the Products table's UnitsInStock column using a COM+ transaction
			SqlCommand inventoryCommand;

			// set up the command's parameter and connection properties
			inventoryCommand = setupInventoryCommand(conn, requestedQty, productID);

			try 
			{

				// update the table
				inventoryCommand.ExecuteNonQuery();

				return true;
			}
		
			finally 
			{

				if ( inventoryCommand != null)
					inventoryCommand.Dispose();
		
			}

		}

												  
		public bool adjustInventory(SqlConnection conn, SqlTransaction trans, int requestedQty, int productID) 
		{

			// this method adjusts the Products table's UnitsInStock column using a SqlTransaction object
			SqlCommand inventoryCommand;

			// set up the command's parameter and connection properties
			inventoryCommand = setupInventoryCommand(conn, requestedQty, productID);

			try 
			{

				// assign the transaction to the command's transaction property. This transaction object is the same 
				// one assigned to the order and orderdetails commands (see newOrder method)
				inventoryCommand.Transaction = trans;

				// update the table
				inventoryCommand.ExecuteNonQuery();

				return true;

			}
			
			finally 
			{

				if ( inventoryCommand != null )
					inventoryCommand.Dispose();
						
			}

		}
		

		public bool checkCreditCard() 
		{

			// Randomly determine validity of credit card 
			Random randomCheck = new Random(DateTime.Now.Millisecond);

			// simulate an invalid credit card 40% of the time
			return (randomCheck.Next(0, 9) < 5);

		}


		private SqlCommand setupNewOrderCommand(SqlConnection conn, string customerID) 
		{

			// set the commandtext, connection and parameter properties for the
			// order command
			SqlCommand orderCommand = new SqlCommand("InsertOrder_sp", conn);

			orderCommand.CommandType = CommandType.StoredProcedure;

			orderCommand.Parameters.Add("@iOrderID", 0);
			orderCommand.Parameters["@iOrderID"].Direction = ParameterDirection.Output;
			orderCommand.Parameters.Add("@cCustomerID", customerID);

			return orderCommand;

		}

		
		private SqlCommand setupNewOrderDetailCommand(SqlConnection conn, Int64 newOrderID, int productID, 
			double unitPrice, int requestedQty, double discount) 
		{

			// set the commandtext, connection and parameter properties for the
			// order details command
			SqlCommand orderDetailCommand = new SqlCommand("AddNewDetail_sp", conn);

			orderDetailCommand.CommandType = CommandType.StoredProcedure;

			orderDetailCommand.Parameters.Add("@iOrderID", newOrderID);
			orderDetailCommand.Parameters.Add("@iProductID", productID);
			orderDetailCommand.Parameters.Add("@mnUnitPrice", unitPrice);
			orderDetailCommand.Parameters.Add("@iQuantity", requestedQty);
			orderDetailCommand.Parameters.Add("@fDiscount", discount);

			return orderDetailCommand;

		}

		
		private SqlCommand setupInventoryCommand(SqlConnection conn, int requestedQty, int productID) 
		{

			// set the commandtext, connection and parameter properties for the
			// products command
			SqlCommand inventoryCommand;
			string sqlString;

			sqlString = "UPDATE Products SET UnitsInStock = UnitsInStock - " + requestedQty + 
				" WHERE ProductID = " + productID;

			inventoryCommand = new SqlCommand(sqlString, conn);

			return inventoryCommand;

		}

	}
}
